#!/bin/bash
#
# The default systemwide limit on USB buffers on Linux is a paltry 16MB[1], shared across all
# running programs. This is insufficient for pushing frames to more than one set of glasses
# simultaneously, so this increases the limit to enable allocating the necessary buffers.
#
# Unfortunately, since this is a systemwide limitation, this cannot account for other USB devices
# on the system that may also have USB buffer requirements, such as high-resolution USB cameras.
# If running into issues with glasses output due to other USB devices, you may need to further
# increase the limit.
#
# [1] https://www.kernel.org/doc/html/v5.15/admin-guide/kernel-parameters.html

DESIRED_USB_MEMORY_MB=${DESIRED_USB_MEMORY_MB:-256}

CURRENT_USB_MEMORY_MB="$(cat /sys/module/usbcore/parameters/usbfs_memory_mb)"

if [[ "$CURRENT_USB_MEMORY_MB" -eq 0 || "$DESIRED_USB_MEMORY_MB" -le "$CURRENT_USB_MEMORY_MB" ]]; then
    # requirement already met: nothing to do
    exit 0
fi

echo "usbcore.usbfs_memory_mb is $CURRENT_USB_MEMORY_MB, changing to $DESIRED_USB_MEMORY_MB"

if ! echo "$DESIRED_USB_MEMORY_MB" > /sys/module/usbcore/parameters/usbfs_memory_mb; then
    echo "failed to update usbcore.usbfs_memory_mb" >&2
    exit 1
fi
